@@ -111,7 +111,7 @@ gs_create(void)
 static void
 cb_sigterm(int sig)
 {
-	exit(255);	// will call cb_atexit()
+	exit(EX_SIGTERM);	// will call cb_atexit()
 }
 
 void
@@ -132,10 +132,44 @@ get_winsize(void)
 	}
 }
 
+// Callback for gs-library to pass log messages to us.
+static void
+cb_gs_log(struct _gs_log_info *l)
+{
+	if (l == NULL)
+		return;
+
+	if (gopt.is_quiet)
+		return;
+
+	// DEBUGF_Y("my level=%d, msg level=%d\n", gopt.verbosity, l->level);
+#ifndef DEBUG
+	// Return if this is _NOT_ a DEBUG-build but we get a TYPE_DEBUG
+	// (should not happen).
+	if (l->type == GS_LOG_TYPE_DEBUG)
+		return;
+#endif
+
+	FILE *fp = gopt.log_fp;
+	if (l->type == GS_LOG_TYPE_ERROR)
+	{
+		fp = gopt.err_fp;
+	}
+
+	if (fp == NULL)
+		return;
+
+	if (l->level > gopt.verbosity)
+		return; // Not interested. 
+
+	fprintf(fp, "%s", l->msg);
+	fflush(fp);
+}
+
 void
 init_vars(void)
 {
-	GS_library_init(gopt.err_fp, /* Debug Output */ gopt.err_fp);
+	GS_library_init(gopt.err_fp, /* Debug Output */ gopt.err_fp, cb_gs_log);
 	GS_LIST_init(&gopt.ids_peers, 0);
 	GS_CTX_init(&gopt.gs_ctx, &gopt.rfd, &gopt.wfd, &gopt.r, &gopt.w, &gopt.tv_now);
 
@@ -160,6 +194,10 @@ init_vars(void)
 	if (gopt.is_multi_peer == 0)
 		GS_CTX_setsockopt(&gopt.gs_ctx, GS_OPT_SINGLESHOT, NULL, 0);
 
+	if (gopt.is_interactive != 0)
+		GS_CTX_setsockopt(&gopt.gs_ctx, GS_OPT_LOW_LATENCY, NULL, 0);
+
+
 	// Prevent startup messages if gs-netcat is started as sub-system from
 	// gs-sftp or gs-mount
 	int is_greetings = 1;
@@ -168,20 +206,19 @@ init_vars(void)
 
 	char *str = getenv("GSOCKET_ARGS");
 	if ((str != NULL) && (strlen(str) > 0) && (is_greetings))
-		VLOG("=Extra arguments: '%s'\n", str);
+		GS_LOG_V("=Extra arguments: '%s'\n", str);
 
 	gopt.sec_str = GS_user_secret(&gopt.gs_ctx, gopt.sec_file, gopt.sec_str);
 	if (gopt.sec_str == NULL)
 		ERREXIT("%s\n", GS_CTX_strerror(&gopt.gs_ctx));
 
 	if (is_greetings)
-		VLOG("=Secret         : \"%s\"\n", gopt.sec_str);
+		GS_LOG("=Secret         : %s\n", gopt.sec_str);
 
 	/* Convert a secret string to an address */
-	GS_ADDR_str2addr(&gopt.gs_addr, gopt.sec_str);
-	gopt.gsocket = gs_create();
+	GS_ADDR_sec2addr(&gopt.gs_addr, gopt.sec_str);
 
-	DEBUGF("PID = %d\n", getpid());
+	GS_LOG_V("=GS Address     : %s\n", GS_addr2hex(NULL, gopt.gs_addr.addr));
 
 	signal(SIGTERM, cb_sigterm);
 }
@@ -201,6 +238,9 @@ usage(const char *params)
 			case 'q':
 				fprintf(stderr, "  -q           Quiet. No log output\n");
 				break;
+			case 'v':
+				fprintf(stderr, "  -v           Verbose. -vv more verbose. -vvv insanely verbose\n");
+				break;
 			case 'r':
 				fprintf(stderr, "  -r           Receive-only. Terminate when no more data.\n");
 				break;
@@ -251,7 +291,7 @@ getcwdx(void)
 {
 #if defined(__sun) && defined(HAVE_OPEN64)
 	// This is solaris 10
-	return getcwd(NULL, PATH_MAX + 1); // solaris10 segfaults if size is 0...
+	return getcwd(NULL, GS_PATH_MAX + 1); // solaris10 segfaults if size is 0...
 #else
 	return getcwd(NULL, 0);
 #endif
@@ -267,6 +307,9 @@ do_getopt(int argc, char *argv[])
 	{
 		switch (c)
 		{
+			case 'v':
+				gopt.verbosity += 1;
+				break;
 			case 'L':
 				gopt.is_logfile = 1;
 				gopt.log_fp = fopen(optarg, "a");
@@ -300,7 +343,7 @@ do_getopt(int argc, char *argv[])
 				 * (at least for the listening server to submit the token securely to the server so that
 				 * the server rejects any listening attempt that uses a bad token)
 				 */
-				VLOG("*** WARNING *** -a not fully supported yet. Trying our best...\n");
+				GS_LOG("*** WARNING *** -a not fully supported yet. Trying our best...\n");
 				gopt.token_str = optarg;
 				break;
 			case 'l':
@@ -320,7 +363,7 @@ do_getopt(int argc, char *argv[])
 				exit(0);
 			case '3':
 				if (strcmp(optarg, "1337") == 0)
-					VLOG("!!Greets to 0xD1G, xaitax and the rest of https://t.me/thcorg!!\n");
+					GS_LOG("!!Greets to 0xD1G, xaitax and the rest of https://t.me/thcorg!!\n");
 		}
 	}
 }
@@ -651,10 +694,6 @@ pty_cmd(const char *cmd, pid_t *pidptr)
 		#ifdef HAVE_FORKPTY
 		fd = open("/dev/tty", O_NOCTTY | O_RDWR);
 		#endif
-		// if (fd >= 0)
-		// {
-		// 	close(fd);
-		// }
 
 		/* HERE: Child */
 		setup_cmd_child(fd);
@@ -913,40 +952,24 @@ cmd_pwd(struct _peer *p)
 	GS_SELECT_FD_SET_W(p->gs);
 }
 
-const char fname_valid_char[] = ""
-"................"
-"................"
-" !.#$%&.()*+,-.."	/* Dont allow " or / or ' */
-"0123456789:;.=.."	/* Dont allow < or > or ? */
-"@ABCDEFGHIJKLMNO"
-"PQRSTUVWXYZ[.]^_"	/* Dont allow \ */
-".abcdefghijklmno"	/* Dont allow ` */
-"pqrstuvwxyz{.}.." 	/* Dont allow | or ~ */
-"";
-
-void
-sanitize_fname_to_str(uint8_t *str, size_t len)
-{
-	int i;
-	uint8_t c;
-
-	for (i = 0; i + 1 < len; i++)
-	{
-		c = str[i];
-		if (c < sizeof fname_valid_char)
-		{
-			if (c == fname_valid_char[c])
-				continue;
-		}
-		if (c == 0)
-			break;
-		DEBUGF("san 0x%02x\n", c);
-		str[i] = '#'; // Change to # if invalid character
-	}
-
-	str[i] = 0x00; // always 0 terminate
-}
-
+/*
+ * Duplicate the process. Parent to check if child dies by monitoring stdin
+ * socketpair to child and parent also monitors its own stdin to
+ * check if calling process has died.
+ *
+ * If child dies then fork again.
+ * This function is different to GS_daemonize
+ * -> ppid is not 1 (not becoming a daemon).
+ * -> not using wait() to check for child's death
+ * -> This parent does not become a new session leader (no setsid()).
+ *
+ * This function is used when gsocket hijacks a process and needs to spawn
+ * a gs-netcat process. The gs-netcat process needs to monitor when the calling
+ * app exits (and this can only be done by monitoring when its own stdin becomes
+ * unavailable).
+ *
+ * This funciton loops forever and never returns.
+ */
 void
 gs_watchdog(void)
 {
@@ -984,7 +1007,7 @@ gs_watchdog(void)
 			{
 				if (errno == EINTR)
 					continue;
-				exit(255); // FATAL
+				exit(EX_BADSELECT); // FATAL
 			}
 
 			// If parent dies then die immediately (this closes fds[0] and all child will die)
