@@ -26,19 +26,17 @@ void format_bps(char *buf, size_t size, int64_t bytes);
 char *getcwdx(void);
 void gs_watchdog(void);
 
-#define VLOG(a...)	do{if (gopt.log_fp != NULL){ fprintf(gopt.log_fp, a); fflush(gopt.log_fp); } }while(0)
+// #define VLOG(a...)	do{if (gopt.log_fp != NULL){ fprintf(gopt.log_fp, a); fflush(gopt.log_fp); } }while(0)
 
 // Log with Timestamp + Peer-ID
-#define VLOG_TSP(_p, _a...)	do{ \
-	if (gopt.log_fp == NULL){ break; } \
-	fprintf(gopt.log_fp, "%s [ID=%d] ", GS_logtime(), _p->id); \
-	fprintf(gopt.log_fp, _a); \
-	fflush(gopt.log_fp); \
+#define GS_LOG_TSP(_p, _a...)	do{ \
+	GS_LOG("%s [ID=%d] ", GS_logtime(), _p->id); \
+	GS_LOG(_a);  \
 }while(0)
 
 /* hack to set rows/columns */
 #define GS_STTY_INIT_HACK	"stty rows %d columns %d\r"
 
-#define UTILS_GETOPT_STR	"3:igqwACTrlSDL:a:s:k:p:d:e:"
+#define UTILS_GETOPT_STR	"3:vigqwACTrlSDL:a:s:k:p:d:e:"
 
 #endif /* !__GSNC_UTILS_H__ */
\ No newline at end of file
